Include "data_barre.geo";


Group {
  Domaine = Region[{AIR,COND}];
  Cond = Region[{COND}];
  Dir = Region[{DIR}];
}

Function {
  mu0 = 4e-7*Pi;
  nu[] = 1./mu0;
  sigma[Cond] = 56e6;
}

Jacobian {
  { Name Vol;
    Case {
      { Region All; Jacobian Vol; }
    }
  }
}

Integration {
  { Name Gauss7;
    Case {
      { Type Gauss;
	Case {
	  { GeoElement Triangle; NumberOfPoints 7; }
	}
      }
    }
  }
}


Constraint {
  { Name Dirichlet;
    Case {
      { Region Dir; Value 0; }
    }
  }
  { Name Courant;
    Case {
      { Region Cond; Value Complex[Ic,0.]; }
    }
  }
}


FunctionSpace {
  // a (2D)
  { Name Hrot_2D; Type Form1P; 
    BasisFunction {
      { Name s_n; NameOfCoef a_n; Function BF_PerpendicularEdge;
	Support Domaine ; Entity NodesOf[All]; }
    }
    Constraint { 
      { NameOfCoef a_n; EntityType NodesOf; NameOfConstraint Dirichlet; }
    }
  }
  
  // grad_v (2D)
  { Name Hregion_u_2D; Type Form1P; // meme que a
    BasisFunction {
      { Name sr; NameOfCoef ur; Function BF_RegionZ;
        // constant vector (over the region) with nonzero z-component only
	Support Cond; Entity Region[{Cond}]; }
    }
    GlobalQuantity {
      { Name U; Type AliasOf; NameOfCoef ur; }
      { Name I; Type AssociatedWith; NameOfCoef ur; }
    }
    Constraint {
      { NameOfCoef I; EntityType Region; NameOfConstraint Courant; }
    }
  }
}


Formulation {
  { Name MagnetoHarmonique_2D; Type FemEquation;    
    Quantity {      
      { Name a; Type Local; NameOfSpace Hrot_2D; } 
      { Name gradv; Type Local; NameOfSpace Hregion_u_2D; }
      { Name I; Type Global; NameOfSpace Hregion_u_2D [I]; }
      { Name U; Type Global; NameOfSpace Hregion_u_2D [U]; }
    }
    Equation {

      Integral { [ nu[] * Dof{d a} , {d a} ];
	In Domaine; Jacobian Vol; Integration Gauss7; }

      // e = -Dt[{a}]-{gradv},
      // with {gradv} = Grad v constant in Cond
      Integral { DtDof [ sigma[] * Dof{a} , {a} ];
        In Cond; Jacobian Vol; Integration Gauss7; }
      Integral { [ sigma[] * Dof{gradv} , {a} ];
	In Cond; Jacobian Vol; Integration Gauss7; }

      // When {gradv} act as a test function, 
      // one obtains the circuits relations,
      // relating the voltage and the current of Cond
      Integral { DtDof [ sigma[] * Dof{a} , {gradv} ];
        In Cond; Jacobian Vol; Integration Gauss7; }
      Integral { [ sigma[] * Dof{gradv} , {gradv} ];
	In Cond; Jacobian Vol; Integration Gauss7; }
      
      GlobalTerm { [ Dof{I} , {U} ]; In Cond; }

    }
  }
}

Resolution {
  { Name MagnetoHarmonique_2D;
    System {
      { Name A; NameOfFormulation MagnetoHarmonique_2D;
	Type ComplexValue; Frequency freq;
      }
    }
    Operation {
      Generate[A];
      Solve[A]; 
      SaveSolution[A];
    }
  }
}

PostProcessing {
  { Name MagnetoHarmonique_2D; NameOfFormulation MagnetoHarmonique_2D;
    PostQuantity {
      { Name a; Value {
	  Term { [ {a} ]; In Domaine; Jacobian Vol; }
        }
      }
      { Name az; Value {
	  Term { [ CompZ[{a}] ]; In Domaine; Jacobian Vol; }
	}
      }
      { Name b; Value {
          Term { [ {d a} ]; In Domaine; Jacobian Vol; }
        }
      }
      { Name norme_b; Value {
          Term { [ Norm[{d a}] ]; In Domaine; Jacobian Vol; }
        }
      }
      { Name h; Value {
          Term { [ nu[] * {d a} ]; In Domaine; Jacobian Vol; }
        }
      }
      { Name j; Value {
	  Term { [ -sigma[] * (Dt[{a}]+{gradv}) ]; In Cond; Jacobian Vol; }
	}
      }
       { Name norm_j; Value {
	   Term { [ Norm[-sigma[] * (Dt[{a}]+{gradv}) ]]; In Cond; Jacobian Vol; }
	}
      }
      { Name Icond_re; Value {
	  Integral { [ Re[CompZ[-sigma[] * (Dt[{a}]+{gradv})]]] ; In Cond; Jacobian Vol; Integration Gauss7;}
	}
      }
      { Name Icond_im; Value {
	  Integral { [ Im[CompZ[-sigma[] * (Dt[{a}]+{gradv})]]] ; 
	    In Cond; Jacobian Vol; Integration Gauss7;}
	}
      }
      { Name gradv; Value {
	  Term { [ {gradv} ]; In Cond; }
	}
      }
      { Name U; Value {
	  Term { [ {U} ]; In Cond; }
	}
      }
      { Name I; Value {
	  Term { [ {I} ]; In Cond; }
	}
      }
      { Name R; Value {
	  Integral { [ SquNorm[-(Dt[{a}]+{gradv}) ]*sigma[]/Ic^2*1e3 ]; 
	    In Cond; Jacobian Vol;  Integration Gauss7;}
	}
      }
    }
  }
}



PostOperation {
  { Name Postproc; NameOfPostProcessing MagnetoHarmonique_2D;
    Operation {
      Print[ j, OnElementsOf Cond, Format Gmsh, File "j.pos" ];
      Print[ b, OnElementsOf Domaine, Format Gmsh, File "b.pos" ];
      Print[ az, OnElementsOf Domaine, Format Gmsh, File "az.pos" ];
      Print[ gradv, OnElementsOf Cond, Format Gmsh, File "gradv.pos" ];
      Print[ Icond_re[Cond], OnGlobal, File  "Icondre.dat",
	SendToServer "Resultats/Courant/1Partie reelle [A]", Color "AliceBlue" ];
       Print[ Icond_im[Cond], OnGlobal, File  "Icondim.dat",
	 SendToServer "Resultats/Courant/2Partie imaginaire [A]", Color "AliceBlue" ]; 
       Print[ I, OnRegion Cond, File "I1.txt", Format Table];
       Print[ norm_j, OnElementsOf Cond, Format Gmsh, File "normj.pos" ];
       Print[ R[Cond], OnGlobal, File  "R.dat",
	 SendToServer "Resultats/Courant/3Resistance [mOhm]", Color "AliceBlue" ];
    }
  }
}
